--Load modules

make_anim = require "make_anim"
set_anim = require "set_anim"
set_annotation = require "set_annotation"
set_thrust = require "set_thrust"
set_feature = require "set_feature"

propeller_anim_state = 0
altimeter_anim_state = 0
tachometer_anim_state = 0
airspeed_anim_state = 0
vertical_speed_anim_state = 0
compass_anim_state = 0

coordinate_hold = false
brake_hold = false

--Vessel properties (J-3)

empty_mass = 350.0 + 100.0 --empty J-3 plus chonky pilot

main_fuel_tank_max = 45.0 -- 12 gallons of gasoline
max_thrust = 1000 --approximate thrust in Newtons
Isp_max = max_thrust*(1000/9.81)

--Wing and airfoil properties (J-3)

wing_area = 16.58 -- sq. meters
wing_span = 10.75 -- meters
wing_effectiveness = 0.70 -- rectangular wing effectiveness (NASA)
wing_aspect_ratio = (wing_span^2) /wing_area
wing_chord_length = wing_area / wing_span

stabilizer_area = 1.36 -- sq. meters
stabilizer_span = 2.90 -- meters
stab_aspect_ratio = (stabilizer_span^2) /stabilizer_area
stab_chord_length = stabilizer_area / stabilizer_span

fin_area = 0.37 -- sq. meters
fin_span = 1.35 -- meters
fin_aspect_ratio = (fin_span^2) /fin_area
fin_chord_length = fin_area / fin_span

elevator_area = 1.0 -- sq. meters
elevator_dcl = 1.5  -- change in lift coefficient due to full deflection

elevatortrim_area = 1.0 -- sq. meters (same as elevator)
elevatortrim_dcl = 1.5  -- change in lift coefficient due to full deflection

aileron_area = 0.9 -- sq. meters
aileron_dcl = 1.5  -- change in lift coefficient due to full deflection

rudder_area = 0.6 -- sq. meters
rudder_dcl = 1.5  -- change in lift coefficient due to full deflection

propeller_radius = 0.9144 --meters
propeller_area = math.pi*propeller_radius*propeller_radius --sq. meters

--Mesh and animation reference points

tail_wheel_axle = {x=0, y=-0.102, z=-4.298}
left_wheel_axle = {x=-0.921, y=-1.126, z=0.87}
right_wheel_axle = {x=0.921, y=-1.126, z=0.87}
main_wheel_radius = 0.2305
tail_wheel_radius = 0.0770

--Points on geometry for collision model

tail_wheel_contact = {x=tail_wheel_axle.x, y=tail_wheel_axle.y-tail_wheel_radius, z=tail_wheel_axle.z}
left_wheel_contact = {x=left_wheel_axle.x, y=left_wheel_axle.y-main_wheel_radius, z=left_wheel_axle.z}
right_wheel_contact = {x=right_wheel_axle.x, y=right_wheel_axle.y-main_wheel_radius, z=right_wheel_axle.z}

nose_tip = {x=0, y=-0.5, z= 2.149}
left_wing_tip = {x=-5.372, y=0.691, z=0}
right_wing_tip = {x=5.372, y=0.691, z=0}
left_elevator_tip = {x=-1.433, y=0.46, z=-3.939}
right_elevator_tip = {x=1.433, y=0.46, z=-3.939}
rudder_tip = {x=0, y=1.177, z=-3.939}

--Points on geometry for flight surface definitions and animations

prop_hub = {x=0, y=0.0, z= 2.149}
lift_point = {x=0,y=0.691,z=0.0}
elevator_pivot = {x=0, y=0.460, z=-3.939}
rudder_pivot = {x=0, y=0, z=-3.939}
left_aileron_pivot = {x=-3.991,y=0.691,z=-0.563}
right_aileron_pivot = {x=3.991,y=0.691,z=-0.563}

--Locations of other animation elements

altimeter_needle_axis = {x=0.094, y=0.143, z=0.894}
vertical_speed_needle_axis = {x=0.188, y=0.143, z=0.894}
airspeed_needle_axis = {x=-0.092, y=0.143, z=0.894}
tachometer_needle_axis = {x=-0.186, y=0.143, z=0.894}
compass_wheel_axis = {x=0.000, y=0.140, z=0.933}

--Center of gravity

cg = {x=0, y=0.0, z=0.0}

function clbk_setclasscaps(cfg)

    vi:set_emptymass(empty_mass)

    set_feature.Set_Contact_Points()

    vi:set_rotdrag({x=0.5, y=0.5, z=0.5})

    vi:create_airfoil(LIFT.VERTICAL,lift_point,'lift', wing_chord_length, wing_area, wing_aspect_ratio)

    vi:create_airfoil(LIFT.VERTICAL,elevator_pivot,'lift_sym', stab_chord_length, stabilizer_area, stab_aspect_ratio)
    vi:create_airfoil(LIFT.HORIZONTAL,rudder_pivot,'lift_sym', fin_chord_length, fin_area, fin_aspect_ratio)

    hmesh = oapi.load_mesh_global('J-3/J-3')

    vi:add_mesh(hmesh)
    vi:set_mesh_visibility_mode(0, MESHVIS.ALWAYS)

    vi:shift_mesh(0,vec.mul(-1,cg))
    vi:set_cameraoffset(vec.add(vec.mul(-1,cg), {x=0.0,y=0.36,z=0.0}))
    vi:set_size(wing_span)

    make_anim.Elevator()

    vi:create_controlsurface(AIRCTRL.ELEVATOR, elevator_area, elevator_dcl, elevator_pivot, AIRCTRL_AXIS.XPOS, 0.3, anim_elevator)
    vi:create_controlsurface(AIRCTRL.ELEVATORTRIM, elevatortrim_area, elevatortrim_dcl, elevator_pivot, AIRCTRL_AXIS.XPOS, 0.3, anim_elevator)

    make_anim.Aileron()

    vi:create_controlsurface(AIRCTRL.AILERON, aileron_area, aileron_dcl, left_aileron_pivot, AIRCTRL_AXIS.XNEG, 0.3, anim_laileron)
    vi:create_controlsurface(AIRCTRL.AILERON, aileron_area, aileron_dcl, right_aileron_pivot, AIRCTRL_AXIS.XPOS, 0.3, anim_raileron)

    make_anim.Rudder()

    vi:create_controlsurface(AIRCTRL.RUDDER, rudder_area, rudder_dcl, rudder_pivot, AIRCTRL_AXIS.YPOS, 0.3, anim_rudder)

    make_anim.Propeller()

    make_anim.Altimeter()
    make_anim.Vertical_Speed()
    make_anim.Airspeed()
    make_anim.Tachometer()
    make_anim.Compass()

    main_fuel_tank = vi:create_propellantresource(main_fuel_tank_max)

    --Following thruster definitions are to separate the user throttle input (th_dummy) from the thrust delivered (th_main)
    --to accomodate future air-breathing reciprocating engine development. Currently the throttle input is simply passed over
    --th_main in the set_thrust.Basic() function call in clbk_poststep. -TC

    th_main = vi:create_thruster({pos=rudder_pivot, dir={x=0, y=0, z=1}, maxth0=max_thrust, hprop=main_fuel_tank, isp0=Isp_max})

    th_dummy = vi:create_thruster({pos=rudder_pivot, dir={x=0, y=0, z=1}, maxth0=1, hprop=main_fuel_tank, isp0=math.huge})
	thg_dummy = vi:create_thrustergroup({th_dummy}, THGROUP.MAIN)


end

function clbk_visualcreated(vis, refcount)

    hdevmesh = vi:get_devmesh(vis,0)

    nvtx = 4
    vtx = oapi.create_ntvertexarray(nvtx)
    vidx = oapi.create_indexarray({0,1,2,3})

    ges = {}
    ges.flags = GRPEDIT.VTXTEXV
    ges.Vtx = vtx
	ges.VIdx = vidx

end

function clbk_poststep(simt,simdt,mjd)

    set_thrust.Basic()

    set_anim.Propeller()
    set_anim.Altimeter()
    set_anim.Vertical_Speed()
    set_anim.Indicated_Airspeed()
    set_anim.Tachometer()
    set_anim.Compass()

    set_feature.Adjust_Contact_Points()
    set_feature.Set_Flying_Tail()
    set_feature.Set_Rolling_Wheels()
    set_feature.Set_Autocoordinate()
    set_feature.Parking_Brake()
    
    --Screen annotations

    --set_annotation.airspeed()
    --set_annotation.altitude()
    --set_annotation.heading()
    --set_annotation.vert_airspeed()
    --set_annotation.coordinate()
    set_annotation.parking_brake()

    if message ~=nil then
        
        set_annotation.message()

    end

end

function clbk_consumebufferedkey(key, down, kstate)

    if not down then -- only process keydown events
        
        return false
    
    end

    if oapi.keydown(kstate, OAPI_KEY.A) then     -- toggle auto-coordinate rudder feature

        if coordinate_hold == false then

            ges.Vtx[1].tv = 1.0
            ges.Vtx[2].tv = 1.0
            ges.Vtx[3].tv = 0.5
            ges.Vtx[4].tv = 0.5

            oapi.edit_meshgroup(hdevmesh, 23, ges)

            coordinate_hold = true

        elseif coordinate_hold == true then

            vi:set_adclevel(AIRCTRL.RUDDER, 0)

            ges.Vtx[1].tv = 0.5
            ges.Vtx[2].tv = 0.5
            ges.Vtx[3].tv = 0.0
            ges.Vtx[4].tv = 0.0

            oapi.edit_meshgroup(hdevmesh, 23, ges)
            
            coordinate_hold = false

        end

        return true

    end

    if oapi.keydown(kstate, OAPI_KEY.B) then     -- apply parking brake

        if brake_hold == false then

            brake_hold = true

        elseif brake_hold == true then
            
            brake_hold = false

        end

        return true

    end

end

function clbk_consumedirectkey(kstate)

    if oapi.keydown(kstate, OAPI_KEY.COMMA) then     -- set left brake

        set_feature.Set_Left_Brake_Force()

    end

    if oapi.keydown(kstate, OAPI_KEY.PERIOD) then     -- set right brake

        set_feature.Set_Right_Brake_Force()

    end

    return false

end

function clbk_loadstateex(scn, vs2)
    
    match = {}

    for line in scenario_lines(scn) do

        if scenario_line_match(line, "coordinate_hold %b", match) then
       
            coordinate_hold = match.res[1]

        elseif scenario_line_match(line, "brake_hold %b", match) then
       
            brake_hold = match.res[1]
        
        else
    
            vi:parse_scenario_line_ex(line, vs2)  -- unrecognised option - pass to Orbiter's generic parser
    
        end
    
    end

end

function clbk_savestate(scn)

    oapi.writescenario_string(scn, "coordinate_hold", tostring(coordinate_hold))
    oapi.writescenario_string(scn, "brake_hold", tostring(brake_hold))

end

function lift(hVessel,aoa,M,Re)
	
    --USA 35B J-3 wing data

    alpha = {-180*RAD, -90*RAD, -46*RAD, -36*RAD, -31*RAD, -26*RAD, -22*RAD, -17*RAD, -12*RAD, -6*RAD, 0*RAD, 5*RAD, 10*RAD, 14*RAD, 17*RAD, 20*RAD, 30*RAD, 90*RAD, 180*RAD}
    cl_a =  {0, 0, 0, -1.4, -1.6, -1.7, -1.6, -1.2, -0.7, 0, 0.7, 1.2, 1.6, 1.7, 1.6, 1.4, 0, 0, 0}

    for i = 1, #alpha-1, 1 do

        if (aoa >= alpha[i]) and (aoa < alpha[i+1]) then 

            cl = cl_a[i] + ((aoa - alpha[i])/(alpha[i+1] - alpha[i]))*(cl_a[i+1]-cl_a[i])

        end

    end

	cm = 0.0
	cd = 0.01 + oapi.get_induceddrag(cl, wing_aspect_ratio, wing_effectiveness)
    
    return cl,cm,cd

end

function lift_sym(hVessel,aoa,M,Re)
	
    --USA 35B J-3 wing data but made symmetric (cl = 0 at aoa = 0) for fin and stabilizer

    alpha = {-180*RAD, -90*RAD, -40*RAD, -30*RAD, -25*RAD, -20*RAD, -16*RAD, -11*RAD, -6*RAD, 0*RAD, 6*RAD, 11*RAD, 16*RAD, 20*RAD, 25*RAD, 30*RAD, 40*RAD, 90*RAD, 180*RAD}
    cl_a =  {0, 0, 0, -1.4, -1.6, -1.7, -1.6, -1.2, -0.7, 0, 0.7, 1.2, 1.6, 1.7, 1.6, 1.4, 0, 0, 0}

    for i = 1, #alpha-1, 1 do

        if (aoa >= alpha[i]) and (aoa < alpha[i+1]) then 

            cl = cl_a[i] + ((aoa - alpha[i])/(alpha[i+1] - alpha[i]))*(cl_a[i+1]-cl_a[i])

        end

    end

	cm = 0.0
	cd = 0.01 + oapi.get_induceddrag(cl, fin_aspect_ratio, wing_effectiveness)
    
    return cl,cm,cd

end