local set_anim = {}

function set_anim.Propeller()

    local throttle = vi:get_thrusterlevel(th_main)

    local rotation_speed = (600 + 2100 * math.sin(throttle * 0.5 * math.pi))*((2*math.pi)/60)

    local propeller_rotation_angle = rotation_speed*oapi.get_simstep()

    propeller_anim_state = (propeller_anim_state + propeller_rotation_angle) % 1

    vi:set_animation(anim_propeller, propeller_anim_state)

end

function set_anim.Altimeter()

    local altitude = vi:get_altitude(ALTMODE.MEANRAD)/0.3048  --altitude converted to feet MSL

    altimeter_anim_state = altitude/10000 % 1

    vi:set_animation(anim_altimeter, altimeter_anim_state)

end

function set_anim.Vertical_Speed()

    local ground_airspeed = vi:get_airspeedvector(REFFRAME.HORIZON)
    local vertical_airspeed = ground_airspeed.y*196.85 --vertical airspeed wrt horizon converted to feet per minute (fpm)

    --Need to interpolate to set the animation level because vertical speed indicator scale is not linear

    local vert_speeds = {-2100, -2000, -1500, -1000, -500, 0, 500, 1000, 1500, 2000, 2100}
    local anim_level = {0.0, 0.02, 0.135, 0.285, 0.415, 0.500, 0.585, 0.715, 0.865, 0.98, 1.0}

    for i = 1, #vert_speeds-1, 1 do

        if (vertical_airspeed >= vert_speeds[i]) and (vertical_airspeed < vert_speeds[i+1]) then 

            vertical_speed_anim_state = anim_level[i] + ((vertical_airspeed - vert_speeds[i])/(vert_speeds[i+1] - vert_speeds[i]))*(anim_level[i+1]-anim_level[i])

        elseif vertical_airspeed > 2100 then

            vertical_speed_anim_state = 1.0

        elseif vertical_airspeed < -2100 then

            vertical_speed_anim_state = 0.0
        
        end

    end

    vi:set_animation(anim_vertical_speed, vertical_speed_anim_state)

end

function set_anim.True_Airspeed()

    local velocity = vi:get_airspeedvector(REFFRAME.LOCAL)

    local airspeed = velocity.z * 2.23694 --airspeed in z-direction, converted to mph

    --Need to interpolate to set the animation level because airspeed indicator scale is not linear

    local speeds = {0, 20, 30, 40, 50, 60, 70, 80, 90, 100, 110, 120, 130, 140, 150}
    local anim_level = {0, 0.02, 0.085, 0.17, 0.245, 0.33, 0.42, 0.51, 0.61, 0.71, 0.79, 0.86, 0.9, 0.95, 1}

    for i = 1, #speeds-1, 1 do

        if (airspeed >= speeds[i]) and (airspeed < speeds[i+1]) then 

            airspeed_anim_state = anim_level[i] + ((airspeed - speeds[i])/(speeds[i+1] - speeds[i]))*(anim_level[i+1]-anim_level[i])

        elseif airspeed > 150 then

            airspeed_anim_state = 1.0

        elseif airspeed < 0 then

            airspeed_anim_state = 0.0

        end

    end

    vi:set_animation(anim_airspeed, airspeed_anim_state)

end

function set_anim.Indicated_Airspeed()

    --Indicated airspeed will differ from true airspeed with altitude because the dynamic pressure that is measured
    --by the Pitot-static system decreases with atmospheric density. Older altimeters would be calibrated for mean sea level.
    --Modern altimeters can be manually adjusted using local atmospheric pressure before take-off.

    local velocity = vi:get_airspeedvector(REFFRAME.LOCAL)
    local density = vi:get_atmdensity()
    local density_msl = 1.225 --kg/m3 at standard sea lebel

    local dynamic_pressure = 0.5*density*velocity.z*velocity.z
    local airspeed = math.sqrt(2*dynamic_pressure/density_msl) * 2.23694 --indicate airspeed in z-direction, converted to mph

    local speeds = {0, 20, 30, 40, 50, 60, 70, 80, 90, 100, 110, 120, 130, 140, 150}
    local anim_level = {0, 0.02, 0.085, 0.17, 0.245, 0.33, 0.42, 0.51, 0.61, 0.71, 0.79, 0.86, 0.9, 0.95, 1}

    for i = 1, #speeds-1, 1 do

        if (airspeed >= speeds[i]) and (airspeed < speeds[i+1]) then 

            airspeed_anim_state = anim_level[i] + ((airspeed - speeds[i])/(speeds[i+1] - speeds[i]))*(anim_level[i+1]-anim_level[i])

        elseif airspeed > 150 then

            airspeed_anim_state = 1.0

        elseif airspeed < 0 then

            airspeed_anim_state = 0.0

        end

    end

    vi:set_animation(anim_airspeed, airspeed_anim_state)

end

function set_anim.Tachometer()

    local throttle = vi:get_thrusterlevel(th_dummy)

    tachometer_anim_state = (600 + 2100 * math.sin(throttle * 0.5 * math.pi))/3000

    vi:set_animation(anim_tachometer, tachometer_anim_state)

end

function set_anim.Compass()

    compass_anim_state = (vi:get_yaw()*DEG)/360

    vi:set_animation(anim_compass, compass_anim_state)

end

return set_anim