local set_annotation = {}

KIAS_annotation = oapi.create_annotation()
KIAS_annotation:set_pos(0.01, 0.22, 0.5, 1)
KIAS_annotation:set_size(0.8)
KIAS_annotation:set_colour({r=1,g=0,b=0})

altitude_annotation = oapi.create_annotation()
altitude_annotation:set_pos(0.01, 0.26, 0.5, 1)
altitude_annotation:set_size(0.8)
altitude_annotation:set_colour({r=1,g=0,b=0})

heading_annotation = oapi.create_annotation()
heading_annotation:set_pos(0.01, 0.30, 0.5, 1)
heading_annotation:set_size(0.8)
heading_annotation:set_colour({r=1,g=0,b=0})

vert_airspeed_annotation = oapi.create_annotation()
vert_airspeed_annotation:set_pos(0.01, 0.34, 0.5, 1)
vert_airspeed_annotation:set_size(0.8)
vert_airspeed_annotation:set_colour({r=1,g=0,b=0})

coord_annotation = oapi.create_annotation()
coord_annotation:set_pos(0.01, 0.38, 0.5, 1)
coord_annotation:set_size(0.8)
coord_annotation:set_colour({r=1,g=0,b=0})

parking_brake_annotation = oapi.create_annotation()
parking_brake_annotation:set_pos(0.01, 0.42, 0.5, 1)
parking_brake_annotation:set_size(0.8)
parking_brake_annotation:set_colour({r=1,g=0,b=0})

message_annotation = oapi.create_annotation()
message_annotation:set_pos(0.01, 0.46, 0.5, 1)
message_annotation:set_size(0.8)
message_annotation:set_colour({r=1,g=0,b=0})

function set_annotation.airspeed()

    velocity = vi:get_airspeedvector(REFFRAME.LOCAL)

    mph = velocity.z*2.23694 --airspeed in z-direction, converted to knots

    -- need to correct for indicated airspeed, just using ktas for now at low altitude

    KIAS_annotation:set_text(string.format("Airspeed %.0f MPH", mph))

end

function set_annotation.altitude()

    altitude = vi:get_altitude(ALTMODE.MEANRAD)/0.3048  --altitude converted to feet MSL
    altitude_annotation:set_text(string.format("Altitude %d feet MSL", altitude))

end

function set_annotation.heading()

    heading = vi:get_yaw()*DEG
    heading_annotation:set_text(string.format("Heading %d deg. true", heading))

end

function set_annotation.vert_airspeed()

    ground_airspeed = vi:get_airspeedvector(REFFRAME.HORIZON)
    vertical_airspeed = ground_airspeed.y*196.85 --vertical airspeed wrt horizon converted to feet per minute (fpm)
    vert_airspeed_annotation:set_text(string.format("Vertical Speed %d fpm", vertical_airspeed))

end

function set_annotation.coordinate()

    if coordinate_hold == true then

        coord_annotation:set_text(string.format("Auto-coordinate (A) ON"))

    else

        coord_annotation:set_text(string.format("Auto-coordinate (A) OFF"))

    end

end

function set_annotation.parking_brake()

    if brake_hold == true then

        parking_brake_annotation:set_text(string.format("Parking Brake (B) ON"))

    else

        parking_brake_annotation:set_text(string.format(""))

    end

end

function set_annotation.message()

    message_annotation:set_text(message)

end

return set_annotation