local set_feature = {}

function set_feature.Set_Contact_Points()

        stiffness_value = 22500
        damping_value = 4000

        td_points = {{pos=tail_wheel_contact, stiffness=stiffness_value, damping=damping_value, mu=0.0, mu_lng=0.0},
        {pos=left_wheel_contact, stiffness=stiffness_value, damping=damping_value, mu=0.0, mu_lng=0.0},
        {pos=right_wheel_contact, stiffness=stiffness_value, damping=damping_value, mu=0.0, mu_lng=0.0},
        {pos=nose_tip, stiffness=stiffness_value, damping=damping_value, mu=0.2, mu_lng=0.2},
        {pos=left_wing_tip, stiffness=stiffness_value, damping=damping_value, mu=0.2, mu_lng=0.2},
        {pos=right_wing_tip, stiffness=stiffness_value, damping=damping_value, mu=0.2, mu_lng=0.2},
        {pos=left_elevator_tip, stiffness=stiffness_value, damping=damping_value, mu=0.2, mu_lng=0.2},
        {pos=right_elevator_tip, stiffness=stiffness_value, damping=damping_value, mu=0.2, mu_lng=0.2},
        {pos=rudder_tip, stiffness=stiffness_value, damping=damping_value, mu=0.2, mu_lng=0.2}}
        
        vi:set_touchdownpoints(td_points)

        --vi:set_touchdownpoints(tail_wheel_contact, left_wheel_contact, right_wheel_contact)

end

function set_feature.Adjust_Contact_Points()

    if vi:get_groundcontact() then
    
        --Following adjusts main contact points to be on wheel surface for general aicraft pitch

        local pitch = vi:get_pitch()

        tail_wheel_contact = {x=tail_wheel_axle.x, y=tail_wheel_axle.y-tail_wheel_radius*math.cos(pitch), z=tail_wheel_axle.z-tail_wheel_radius*math.sin(pitch)}
        left_wheel_contact = {x=left_wheel_axle.x, y=left_wheel_axle.y-main_wheel_radius*math.cos(pitch), z=left_wheel_axle.z-main_wheel_radius*math.sin(pitch)}
        right_wheel_contact = {x=right_wheel_axle.x, y=right_wheel_axle.y-main_wheel_radius*math.cos(pitch), z=right_wheel_axle.z-main_wheel_radius*math.sin(pitch)}

        set_feature.Set_Contact_Points()

    end

end

function set_feature.Set_Flying_Tail()

    --Following is meant to mimic a flying tail (deflection of prop wash by tail surfaces)

    local rudder_input = vi:get_adclevel(AIRCTRL.RUDDER)
    local elevator_input = vi:get_adclevel(AIRCTRL.ELEVATOR)
    
    local Dx = math.cos(33*RAD*0.5*elevator_input)*math.sin(-33*RAD*0.5*rudder_input)
    local Dy = -math.sin(33*RAD*0.5*elevator_input)
    local Dz = math.cos(33*RAD*0.5*elevator_input)*math.cos(-33*RAD*0.5*rudder_input)

    vi:set_thrusterdir(th_main,{x=Dx,y=Dy,z=Dz})

end

function set_feature.Set_Rolling_Wheels()

    --Following is to mimic realistic rolling tires without side scrub

    if vi:get_groundcontact() then

        local groundspeed = vi:get_groundspeedvector(REFFRAME.LOCAL)
        local side_slip = groundspeed.x
        local mass = empty_mass + vi:get_propellantmass(main_fuel_tank)
        local simdt =  oapi.get_simstep()

        side_force_new = mass * side_slip /simdt

        if side_force_old == nil then

            side_force_old = side_force_new

        end

        --Need to under-relax the force calculation so vessel doesn't get yeeted to Alpha Quadrant

        side_force = side_force_old + 0.5*(side_force_new - side_force_old)

        side_force_old = side_force
        
        vi:add_force({x=-side_force, y=0, z=0}, {x=0,y=right_wheel_contact.y,z=right_wheel_contact.z})

    else

        side_force_old = 0
        
    end

end

function set_feature.Set_Autocoordinate()

    if coordinate_hold == true then
        
        local V = vi:get_airspeedvector(REFFRAME.LOCAL)

        if vi:get_airspeed() == 0 then
    
            yaw_error = 0
    
        else
    
            yaw_error = math.asin(V.x/math.sqrt(V.x*V.x + V.z*V.z))
    
        end
    
        Kp = 100
    
        local rudder_input = Kp * yaw_error
    
        vi:set_adclevel(AIRCTRL.RUDDER, rudder_input)

    end

end

function set_feature.Apply_Brake_Force()
    
    local groundspeed = vi:get_groundspeedvector(REFFRAME.LOCAL)
    local forward_speed = groundspeed.z
    local mass = empty_mass + vi:get_propellantmass(main_fuel_tank)
    local simdt =  oapi.get_simstep()

    if forward_speed == 0 then --set brake force to counter thrust parallel to ground

        F = vi:get_thrustvector(th_main)
        pitch = vi:get_pitch()

        brake_force = 0.5*F.z*math.cos(pitch)

    else

        if math.abs(forward_speed/simdt) > 9.81 then

            --determine if velocity is positive or negative
                
            if forward_speed > 0 then
                
                brake_force = 0.5*mass*9.81

            else

                brake_force = -0.5*mass*9.81

            end

        else

            brake_force = 0.5*mass*forward_speed/simdt
            
        end

    end

    return brake_force

end

function set_feature.Set_Left_Brake_Force()

    if vi:get_groundcontact() then
        brake_force = set_feature.Apply_Brake_Force()
        vi:add_force({x=0, y=0, z=-brake_force}, left_wheel_contact)
    end

    return true

end

function set_feature.Set_Right_Brake_Force()

    if vi:get_groundcontact() then
        brake_force = set_feature.Apply_Brake_Force()
        vi:add_force({x=0, y=0, z=-brake_force}, right_wheel_contact)
    end

    return true

end

function set_feature.Parking_Brake()

    if brake_hold == true then
    
        set_feature.Set_Left_Brake_Force()
        set_feature.Set_Right_Brake_Force()

    end

end

return set_feature